unit EditMan;

{
Copyright 1996 by Justin Turberville
Address: P.O.Box 122
         Kenton-On-Sea
         6191
         South Africa
EMail: justint@cyberjoe.co.za

Links and manages the edit controls associated with a dataset.
Assign a handler to OnEditClick - TEditManager automatically assigns the  
handler to all attached edit control OnClick properties. In this handler,
call the edit manager's HandleEdit method, passing the sender. Use the 
OnUpdateControls event to do any housework.
TEditManager can generate an edit menu, popup menu edit items and edit 
speedbuttons at design time - save you about 3 minutes!

Let me know if you find this useful/improve on it!

Sorry about leaving out AddCtrls.res last time!
And for the mess on the test form.
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DB, Buttons, Menus, StdCtrls, DBTables, DsgnIntf;

const
  erInsert = 0;
  erDelete = 1;
  erEdit   = 2;
  erPost   = 3;
  erCancel = 4;

type
  TEditManager = class;

  TEditCtrls = array[erInsert .. erCancel] of TComponent;

  TEditRefs = class(TPersistent)
  private
    FManager: TEditManager;
    FRefs: TEditCtrls;
    function GetRefs(Index: Integer): TComponent;
    procedure SetRefs(Index: Integer; Value: TComponent);
  protected
    procedure UpdateRef(Ref: TComponent; Value: Boolean); virtual; abstract;
    procedure SetValue(Index: Integer; Value: TComponent); virtual;
  public
    constructor Create(AOwner: TEditManager);
    procedure UpdateRefs(Active, Editing, Empty: Boolean);
    procedure RemoveRef(AComponent: TComponent);
    function GetRefIndex(AComponent: TComponent): Integer;
    property Refs[Index: Integer]: TComponent read GetRefs write SetRefs;
  end;

  TEditBtnRefs = class(TEditRefs)
  private
    function GetInsertBtn: TSpeedButton;
    procedure SetInsertBtn(Value: TSpeedButton);
    function GetDeleteBtn: TSpeedButton;
    procedure SetDeleteBtn(Value: TSpeedButton);
    function GetEditBtn: TSpeedButton;
    procedure SetEditBtn(Value: TSpeedButton);
    function GetPostBtn: TSpeedButton;
    procedure SetPostBtn(Value: TSpeedButton);
    function GetCancelBtn: TSpeedButton;
    procedure SetCancelBtn(Value: TSpeedButton);
    procedure SetValue(Index: Integer; Value: TComponent); override;
  protected
    procedure UpdateRef(Ref: TComponent; Value: Boolean); override;
  public
    procedure SetOnClicks;
  published
    property InsertBtn: TSpeedButton read GetInsertBtn write SetInsertBtn;
    property DeleteBtn: TSpeedButton read GetDeleteBtn write SetDeleteBtn;
    property EditBtn:   TSpeedButton read GetEditBtn   write SetEditBtn;
    property PostBtn:   TSpeedButton read GetPostBtn   write SetPostBtn;
    property CancelBtn: TSpeedButton read GetCancelBtn write SetCancelBtn;
  end;

  TEditMenuRefs = class(TEditRefs)
  private
    function GetInsertItem: TMenuItem;
    procedure SetInsertItem(Value: TMenuItem);
    function GetDeleteItem: TMenuItem;
    procedure SetDeleteItem(Value: TMenuItem);
    function GetEditItem: TMenuItem;
    procedure SetEditItem(Value: TMenuItem);
    function GetPostItem: TMenuItem;
    procedure SetPostItem(Value: TMenuItem);
    function GetCancelItem: TMenuItem;
    procedure SetCancelItem(Value: TMenuItem);
    procedure SetValue(Index: Integer; Value: TComponent); override;
  protected
    procedure UpdateRef(Ref: TComponent; Value: Boolean); override;
  public
    procedure SetOnClicks;
  published
    property InsertItem: TMenuItem read GetInsertItem write SetInsertItem;
    property DeleteItem: TMenuItem read GetDeleteItem write SetDeleteItem;
    property EditItem:   TMenuItem read GetEditItem   write SetEditItem;
    property PostItem:   TMenuItem read GetPostItem   write SetPostItem;
    property CancelItem: TMenuItem read GetCancelItem write SetCancelItem;
  end;

  TEditDataLink = class;

  TEditManager = class(TComponent)
  private
    FAbout: string;
    FDataLink: TEditDataLink;
    FSpeedButtons: TEditBtnRefs;
    FMenuItems, FPopupItems: TEditMenuRefs;
    FManageCtrls: Boolean;
    FConfirmDelete: Boolean;
    FOnEditClick: TNotifyEvent;
    FOnUpdateCtrls: TNotifyEvent;
    function GetDataSource: TDataSource;
    procedure SetDataSource(Value: TDataSource);
    procedure SetManageCtrls(Value: Boolean);
    procedure SetOnEditClick(Value: TNotifyEvent);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation);
      override;
    procedure HandleEdit(Sender: TObject);
    procedure UpdateCtrls;
    function GetCtrlIndex(AObject: TObject): Integer;
  published
    property About: string read FAbout write FAbout;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property SpeedButtons: TEditBtnRefs read FSpeedButtons write FSpeedButtons;
    property MenuItems: TEditMenuRefs read FMenuItems write FMenuItems;
    property PopupItems: TEditMenuRefs read FPopupItems write FPopupItems;
    property ManageCtrls: Boolean read FManageCtrls write SetManageCtrls
      default True;
    property ConfirmDelete: Boolean read FConfirmDelete write FConfirmDelete
      default False;
    property OnEditClick: TNotifyEvent read FOnEditClick write SetOnEditClick;
    property OnUpdateCtrls: TNotifyEvent read FOnUpdateCtrls write FOnUpdateCtrls;
  end;

  TEditDataLink = class(TDataLink)
  private
    FManager: TEditManager;
  protected
    procedure EditingChanged; override;
    procedure DataSetChanged; override;
    procedure ActiveChanged; override;
  public
    constructor Create(AManager: TEditManager);
    destructor Destroy; override;
    function RecordCount: Integer;
  end;

  TEditRefsProperty = class(TClassProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
  end;

procedure Register;

implementation

uses
  AddCtrls;

procedure Register;
begin
  RegisterComponents('Samples', [TEditManager]);
  RegisterComponentEditor(TEditManager, TAddControlsEditor);
  RegisterPropertyEditor(TypeInfo(TEditRefs), TEditManager, '',
    TEditRefsProperty);
end;

{TEditRefs}

constructor TEditRefs.Create(AOwner: TEditManager);
var
  i: Integer;
begin
  inherited Create;
  FManager := AOwner;
  for i := erInsert to erCancel do
    FRefs[i] := nil;
end;

function TEditRefs.GetRefIndex(AComponent: TComponent): Integer;
var
  i: Integer;
begin
  Result := -1;
  i := erInsert;
  while (Result = -1) and (i <= erCancel) do
    if FRefs[i] = AComponent then
      Result := i else
      Inc(i);
end;

procedure TEditRefs.UpdateRefs(Active, Editing, Empty: Boolean);
begin
  UpdateRef(FRefs[erInsert], Active and not Editing);
  UpdateRef(FRefs[erDelete], Active and not (Editing or Empty));
  UpdateRef(FRefs[erEdit],   Active and not (Editing or Empty));
  UpdateRef(FRefs[erPost],   Editing);
  UpdateRef(FRefs[erCancel], Editing);
end;

procedure TEditRefs.RemoveRef(AComponent: TComponent);
var
  i: Integer;
begin
  for i := erInsert to erCancel do
    if FRefs[i] = AComponent then
      FRefs[i] := nil;
end;

procedure TEditRefs.SetValue(Index: Integer; Value: TComponent);
begin
  FRefs[Index] := Value;
end;

function TEditRefs.GetRefs(Index: Integer): TComponent;
begin
  Result := FRefs[Index];
end;

procedure TEditRefs.SetRefs(Index: Integer; Value: TComponent);
begin
  SetValue(Index, Value);
end;

{TEditBtnRefs}

function TEditBtnRefs.GetInsertBtn: TSpeedButton;
begin
  Result := TSpeedButton(FRefs[erInsert]);
end;

procedure TEditBtnRefs.SetInsertBtn(Value: TSpeedButton);
begin
  SetValue(erInsert, Value);
end;

function TEditBtnRefs.GetDeleteBtn: TSpeedButton;
begin
  Result := TSpeedButton(FRefs[erDelete]);
end;

procedure TEditBtnRefs.SetDeleteBtn(Value: TSpeedButton);
begin
  SetValue(erDelete, Value);
end;

function TEditBtnRefs.GetEditBtn: TSpeedButton;
begin
  Result := TSpeedButton(FRefs[erEdit]);
end;

procedure TEditBtnRefs.SetEditBtn(Value: TSpeedButton);
begin
  SetValue(erEdit, Value);
end;

function TEditBtnRefs.GetPostBtn: TSpeedButton;
begin
  Result := TSpeedButton(FRefs[erPost]);
end;

procedure TEditBtnRefs.SetPostBtn(Value: TSpeedButton);
begin
  SetValue(erPost, Value);
end;

function TEditBtnRefs.GetCancelBtn: TSpeedButton;
begin
  Result := TSpeedButton(FRefs[erCancel]);
end;

procedure TEditBtnRefs.SetCancelBtn(Value: TSpeedButton);
begin
  SetValue(erCancel, Value);
end;

procedure TEditBtnRefs.SetOnClicks;
var
  i: Integer;
begin
  for i := erInsert to erCancel do
    if FRefs[i] <> nil then
      TSpeedButton(FRefs[i]).OnClick := FManager.OnEditClick;
end;

procedure TEditBtnRefs.SetValue(Index: Integer; Value: TComponent);
begin
  if FRefs[Index] <> Value then begin
    if FRefs[Index] <> nil then
      TSpeedButton(FRefs[Index]).OnClick := nil;
    if Value <> nil then begin
      TSpeedButton(Value).OnClick := FManager.OnEditClick;
      TSpeedButton(Value).Enabled := False;
    end;
    inherited;
  end;
end;

procedure TEditBtnRefs.UpdateRef(Ref: TComponent; Value: Boolean);
begin
  if Ref <> nil then TSpeedButton(Ref).Enabled := Value;
end;

{TEditMenuRefs}

function TEditMenuRefs.GetInsertItem: TMenuItem;
begin
  Result := TMenuItem(FRefs[erInsert]);
end;

procedure TEditMenuRefs.SetInsertItem(Value: TMenuItem);
begin
  SetValue(erInsert, Value);
end;

function TEditMenuRefs.GetDeleteItem: TMenuItem;
begin
  Result := TMenuItem(FRefs[erDelete]);
end;

procedure TEditMenuRefs.SetDeleteItem(Value: TMenuItem);
begin
  SetValue(erDelete, Value);
end;

function TEditMenuRefs.GetEditItem: TMenuItem;
begin
  Result := TMenuItem(FRefs[erEdit]);
end;

procedure TEditMenuRefs.SetEditItem(Value: TMenuItem);
begin
  SetValue(erEdit, Value);
end;

function TEditMenuRefs.GetPostItem: TMenuItem;
begin
  Result := TMenuItem(FRefs[erPost]);
end;

procedure TEditMenuRefs.SetPostItem(Value: TMenuItem);
begin
  SetValue(erPost, Value);
end;

function TEditMenuRefs.GetCancelItem: TMenuItem;
begin
  Result := TMenuItem(FRefs[erCancel]);
end;

procedure TEditMenuRefs.SetCancelItem(Value: TMenuItem);
begin
  SetValue(erCancel, Value);
end;

procedure TEditMenuRefs.SetOnClicks;
var
  i: Integer;
begin
  for i := erInsert to erCancel do
    if FRefs[i] <> nil then
      TMenuItem(FRefs[i]).OnClick := FManager.OnEditClick;
end;

procedure TEditMenuRefs.SetValue(Index: Integer; Value: TComponent);
begin
  if FRefs[Index] <> Value then begin
    if FRefs[Index] <> nil then
      TMenuItem(FRefs[Index]).OnClick := nil;
    if Value <> nil then begin
      TMenuItem(Value).OnClick := FManager.OnEditClick;
      TMenuItem(Value).Enabled := False;
    end;
    inherited;
  end;
end;

procedure TEditMenuRefs.UpdateRef(Ref: TComponent; Value: Boolean);
begin
  if Ref <> nil then TMenuItem(Ref).Enabled := Value;
end;

{TEditManager}

constructor TEditManager.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FAbout := 'v1.0 by J.Turberville - justint@cyberjoe.co.za';
  FSpeedButtons := TEditBtnRefs.Create(Self);
  FMenuItems    := TEditMenuRefs.Create(Self);
  FPopupItems   := TEditMenuRefs.Create(Self);
  FDataLink     := TEditDataLink.Create(Self);
  FManageCtrls  := True;
end;
                                                      
destructor TEditManager.Destroy;
begin
  FDataLink.Free;
  inherited Destroy;
end;

procedure TEditManager.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
    if AComponent is TSpeedButton then
      FSpeedButtons.RemoveRef(AComponent)
    else if AComponent is TMenuItem then begin
      FMenuItems.RemoveRef(AComponent);
      FPopupItems.RemoveRef(AComponent);
    end;
end;

function TEditManager.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TEditManager.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  UpdateCtrls;
end;

procedure TEditManager.SetManageCtrls(Value: Boolean);
begin
  FManageCtrls := Value;
  UpdateCtrls;
end;

procedure TEditManager.HandleEdit(Sender: TObject);
var
  i: Integer;
  b: Boolean;
begin
  if not FDataLink.Active then Exit;
  i := GetCtrlIndex(Sender);
  with FDataLink.DataSource.DataSet do
    case i of
      erInsert: Insert;
      erDelete: begin
        if FConfirmDelete then b := MessageDlg('Delete this record?',
          mtConfirmation, [mbYes, mbNo], 0) = mrYes else b := True;
        if b then Delete;
      end;
      erEdit:   Edit;
      erPost:   Post;
      erCancel: Cancel;
    end;
end;

procedure TEditManager.UpdateCtrls;
begin
  if FManageCtrls then
    with FDataLink do begin
      FSpeedButtons.UpdateRefs(Active, Editing, RecordCount = 0);
      FMenuItems.UpdateRefs(Active, Editing, RecordCount = 0);
      FPopupItems.UpdateRefs(Active, Editing, RecordCount = 0);
      if Assigned(FOnUpdateCtrls) then FOnUpdateCtrls(Self);
    end;
end;

function TEditManager.GetCtrlIndex(AObject: TObject): Integer;
begin
  Result := -1;
  if AObject is TSpeedButton then
    Result := FSpeedButtons.GetRefIndex(TSpeedButton(AObject))
  else if AObject is TMenuItem then begin
    Result := FMenuItems.GetRefIndex(TMenuItem(AObject));
    if Result = -1 then
      Result := FPopupItems.GetRefIndex(TMenuItem(AObject));
  end;
end;

procedure TEditManager.SetOnEditClick(Value: TNotifyEvent);
begin
  FOnEditClick := Value;
  FSpeedButtons.SetOnClicks;
  FMenuItems.SetOnClicks;
  FPopupItems.SetOnClicks;
end;

{TEditDataLink}

constructor TEditDataLink.Create(AManager: TEditManager);
begin
  inherited Create;
  FManager := AManager;
end;

destructor TEditDataLink.Destroy;
begin
  FManager := nil;
  inherited Destroy;
end;

function TEditDataLink.RecordCount: Integer;
begin
  if (DataSource = nil) or not Active then       {causes an access error!}
    Result := 0 else
    Result := inherited RecordCount;
end;

procedure TEditDataLink.EditingChanged;
begin
  if FManager <> nil then FManager.UpdateCtrls;
end;

procedure TEditDataLink.DataSetChanged;
begin
  if FManager <> nil then FManager.UpdateCtrls;
end;

procedure TEditDataLink.ActiveChanged;
begin
  if FManager <> nil then FManager.UpdateCtrls;
end;

{TEditRefsProperty}

function TEditRefsProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paSubProperties];
end;

end.
